/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <nostalgia/core/gfx.hpp>

#include "scene.hpp"

namespace nostalgia::scene {

Scene::Scene(const SceneStatic &sceneStatic) noexcept:
	m_sceneStatic(sceneStatic) {
}

ox::Error Scene::setupDisplay(core::Context *ctx) noexcept {
	if (m_sceneStatic.palettes.empty()) {
		return OxError(1, "Scene has no palettes");
	}
	const auto &palette = m_sceneStatic.palettes[0];
	oxReturnError(core::loadBgTileSheet(
				ctx, 0, m_sceneStatic.tilesheet, palette));
	// disable all backgrounds
	core::setBgStatus(ctx, 0);
	for (auto layerNo = 0u; const auto &layer : m_sceneStatic.tileMapIdx) {
		core::setBgStatus(ctx, layerNo, true);
		core::setBgCbb(ctx, layerNo, 0);
		auto x = 0;
		auto y = 0;
		auto width = m_sceneStatic.rows[layerNo];
		for (const auto &tile : layer) {
			core::setTile(ctx, layerNo, x, y, tile);
			core::setTile(ctx, layerNo, x + 1, y, tile + 1);
			core::setTile(ctx, layerNo, x, y + 1, tile + 2);
			core::setTile(ctx, layerNo, x + 1, y + 1, tile + 3);
			x += 2;
			if (x >= width * 2) {
				x = 0;
				y += 2;
			}
		}
		++layerNo;
	}
	return {};
}

}
