/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include "sceneeditorview.hpp"

namespace nostalgia::scene {

SceneEditorView::SceneEditorView(core::Context *ctx, const SceneStatic &sceneStatic) noexcept:
	m_ctx(*ctx),
	m_sceneStatic(sceneStatic),
	m_scene(m_sceneStatic) {
}

void SceneEditorView::setupScene() noexcept {
	oxIgnoreError(m_scene.setupDisplay(&m_ctx));
}

void SceneEditorView::draw(int width, int height) noexcept {
	if (width != m_frameBuffer.width || height != m_frameBuffer.height) {
		glutils::resizeInitFrameBuffer(&m_frameBuffer, width, height);
		core::gl::setRenderSize(&m_ctx, width, height);
		oxIgnoreError(m_scene.setupDisplay(&m_ctx));
	}
	glBindFramebuffer(GL_FRAMEBUFFER, m_frameBuffer);
	glViewport(0, 0, m_frameBuffer.width, m_frameBuffer.height);
	// draw begin
	core::gl::drawMainView(&m_ctx);
	// draw end
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
}

const glutils::FrameBuffer &SceneEditorView::framebuffer() const noexcept {
	return m_frameBuffer;
}

}
